<?php
/**
 * Functions to register client-side assets (scripts and stylesheets) for the
 * Gutenberg block.
 *
 * @package event-post
 */

/**
 * Registers all block assets so that they can be enqueued through Gutenberg in
 * the corresponding context.
 *
 * @see https://wordpress.org/gutenberg/handbook/blocks/writing-your-first-block-type/#enqueuing-block-scripts
 * @since 5.2
 */
function eventpost_timeline_block_init() {
	global $EventPost;
	$dir_path = $EventPost->plugin_path;
	$dir = dirname( __DIR__ );

	$block_js = 'js/block-timeline.min.js';
	wp_register_script(
		'event-post-timeline-block-editor-script',
		plugins_url( $block_js, $dir ),
		array(
			'wp-blocks',
			'wp-editor',
			'wp-components',
			'wp-i18n',
			'wp-element',
		),
		filemtime( "$dir_path/$block_js" )
	);


	wp_register_script(
		'event-post-timeline-block-editor-script-front-end',
		plugins_url( 'js/event-timeline.min.js', $dir ),
		array('jquery',),
		filemtime( "$dir_path/js/event-timeline.min.js" )
	);

	wp_add_inline_script( 'event-post-timeline-block-editor-script-front-end', 'const event_post_ajaxurl = "'.admin_url( 'admin-ajax.php' ).'";' );
	wp_set_script_translations( 'event-post-timeline-block-editor-script', 'event-post' );
	$block_css = 'css/block-timeline.css';
	wp_register_style(
		'event-post-timeline-block-editor-style',
		plugins_url( $block_css, $dir ),
		array(),
		filemtime( "$dir_path/$block_css" )
	);

	register_block_type( 'eventpost/timeline', array(
		'editor_script' => 'event-post-timeline-block-editor-script',
		'editor_style'  => 'event-post-timeline-block-editor-style',
		'style'  => 'event-post-timeline-block-editor-style',
		'script'  => 'event-post-timeline-block-editor-script-front-end',
		'render_callback' => array($EventPost->Shortcodes, 'shortcode_timeline'),
		'textdomain' => "event-post",
	) );
}
add_action( 'init', 'eventpost_timeline_block_init' );
